/************************************************************************

    2048: Sliding-block puzzle game 

    Copyright (C) 2014  John Elliott <jce@seasip.demon.co.uk>
                        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
        
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

/* Test harness for the 2048 game engine -- before I'd created the GEM
 * front end, I used this DOS/UNIX console program to test its logic */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#ifdef __MSDOS__
#include <conio.h>
#include <dos.h>
#else
#include <unistd.h>
#include <termios.h>
#endif
#include "2048game.h"

GAMEBOARD st_game;

#ifdef __MSDOS__

/* Under DOS, clear screen by getting the current video mode and setting it 
 * again */
void cls(void)
{
	union REGS rg;

	rg.x.ax = 0x0F00;
	int86(0x10, &rg, &rg);
	rg.h.ah = 0;
	int86(0x10, &rg, &rg);
}

#else

/* UNIX has no builtin getch(), but termios provides */
char getch(void)
{
	struct termios ts, ots;
        char c;
	int i;

	int filen = fileno(stdin);
        tcgetattr(filen, &ts);		/* Switch the terminal to raw mode */
        tcgetattr(filen, &ots);
        cfmakeraw(&ts);
        tcsetattr(filen, TCSANOW, &ts);
        
        fflush(stdout);
        fflush(stdin);
	do
	{
		i = read(filen, &c, 1);
	}
	while (i < 0);
        
        tcsetattr(filen, TCSANOW, &ots);

	fflush(stdin);
	return c;
}

/* Rather than do the right thing and link with ncurses, I hardcoded ESC [ 2 J 
 * as the clear screen sequence. Naughty. */
void cls()
{
	printf("\033[2J");
	fflush(stdout);
}

#endif



/* Draw the board and curent score */
void c_draw(GAMEBOARD *g)
{
	int r, c;

	cls();
	printf("Score: %ld\n", g->score);
	for (r = 1; r <= 4; r++)
	{
		for (c = 1; c <= 4; c++)
		{
/* Newly-created tiles will have * * each side of their number */
			if (g->tile[r][c] & MERGED)
			{
				printf("[*%4d*]", g->tile[r][c] & ~MERGED);
			}
			else
			{
				printf("[ %4d ]", g->tile[r][c]);
			}
		}
		putchar('\n');
	}
}

/* Main loop: */
int main(int argc, char **argv)
{
	time_t t;
	int r;

	/* Initialise random generator */
	time(&t);
	srand(t);

	/* Create empty board */
	gb_init(&st_game);
	/* Drop two tiles onto it */
	gb_newtile(&st_game);
	gb_newtile(&st_game);

	while (1)	/* Main loop: */
	{
		/* Draw the board */
		c_draw(&st_game);

		/* If no moves possible, abort */
		if (!gb_canmove(&st_game))
		{
			fprintf(stderr, "Game over: No moves possible.\n");
			return 1;
		}
		/* Use vi keys h/j/k/l to move */
		switch (getch())
		{
			case 'h': case 'H': r = gb_move(&st_game, -1, 0); break;
			case 'j': case 'J': r = gb_move(&st_game, 0, 1); break;
			case 'k': case 'K': r = gb_move(&st_game, 0, -1); break;
			case 'l': case 'L': r = gb_move(&st_game, 1, 0); break;
			case 'q': case 'Q': return 1;
		}
		/* Check for victory */
		if (r == 2)
		{
			fprintf(stderr, "You have won!");
			return 0;
		}
		/* If a move was successful, drop a new tile onto the board */
		if (r == 1)
		{
			gb_newtile(&st_game);
		}
	}

	return 0;
}
